import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { OrderServiceRepository } from "./order-service.repository";
import { CreateOrderServiceDto } from "./dto/create-order-service.dto";
import { OrderService } from "./entities/order-service.entity";
import { getGraphQLClient } from "../../helper/graphql-client-helper";
import { fetchMOWithOrderNumber } from "../../helper/dutchie-plus-query";

@Injectable()
export class OrderServiceService {
    constructor(
      @InjectRepository(OrderServiceRepository)
      private readonly repository: OrderServiceRepository
    ) {
    }

    async create(
      dto: CreateOrderServiceDto
    ): Promise<OrderService | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async findAll(): Promise<OrderService[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<OrderService[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async getOrderUserWise(variables: any): Promise<any | Error> {
        try {
            const data = await this.repository.findByFilterQuery({ email: variables.email });
            if (!data) {
                throw new HttpException(
                  Constants.NOT_FOUND,
                  HttpStatus.NOT_FOUND
                );
            }
            const graphQLClient = getGraphQLClient();
            let allAPI = [];
            for (let i = 0; i < Object.keys(data).length; i++) {
                const cusVariables = {
                    retailerId: variables.retailerId,
                    filter: {
                        orderNumber: data[i].orderNumber
                    }
                };
                const query = fetchMOWithOrderNumber();
                const response = graphQLClient.request(query, cusVariables);
                allAPI.push(response.catch((errors: any) => {
                    console.log(errors);
                    return Promise.resolve();
                }));
            }
            let finalResponse = [];
            await Promise.all(allAPI)
              .then((results: any) => {
                  for (let i = 0; i < results.length; i++) {
                      const res = results[i].orders.length > 0 ? results[i].orders[0] : null;
                      if(res !== null) {
                          finalResponse.push(res);
                      }
                  }
              })
              .catch(error => {
                  console.log(error);
              })
              .finally(() => {
                  finalResponse.sort((a: any, b: any) => {
                      if (a.createdAt > b.createdAt) {
                          return -1;
                      }
                      if (a.createdAt < b.createdAt) {
                          return 1;
                      }
                      return 0;
                  });
                  return finalResponse;
              });

            return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", finalResponse);
        } catch (e) {
            return e;
        }
    }
}
